@@ -1,13 +1,14 @@
 package Spreadsheet::HTML;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 use Clone;
 use HTML::Element;
 use Math::Matrix;
 
 use Spreadsheet::HTML::CSV;
+use Spreadsheet::HTML::XLS;
 use Spreadsheet::HTML::HTML;
 use Spreadsheet::HTML::JSON;
 use Spreadsheet::HTML::YAML;
@@ -43,32 +44,33 @@ sub reverse   {
 sub process {
     my ($self,$data,$args) = _args( @_ );
 
-    # return if cached
-    if (ref($self) and $self->{is_cached}) {
+    if ($self and $self->{is_cached}) {
         return wantarray ? ( data => $self->{data}, %$args ) : $data;
     }
 
+    my $empty = exists $args->{empty} ? $args->{empty} : '&nbsp;';
     my $max_cols = scalar @{ $data->[0] };
 
-    # assign each cell as a td first
-    my $first = 1;
-    for my $row (@{$data}) {
+    for my $row (0 .. $#$data) {
 
-        # pad and truncate
-        push @$row, undef for 1 .. ($max_cols - scalar @$row);
-        $row = [ @$row[0 .. $max_cols - 1] ] if scalar( @$row ) > $max_cols;
+        push @{ $data->[$row] }, undef for 1 .. $max_cols - $#{ $data->[$row] } + 1;  # pad
+        pop  @{ $data->[$row] } for $max_cols .. $#{ $data->[$row] };                 # truncate
 
-        for my $i (0 .. $#$row) {
-            $row->[$i] = ($first and !($args->{headless} or $args->{matrix}))
-                ? _element( th => _scrub( $row->[$i] ), $args->{th} )
-                : _element( td => _scrub( $row->[$i] ), $args->{td} )
-            ;
+        for my $col (0 .. $#{ $data->[$row] }) {
+            my $tag = (!$row and !($args->{headless} or $args->{matrix})) ? 'th' : 'td';
+            my $val = $data->[$row][$col];
+
+            # --empty
+            do{ no warnings; $val =~ s/^\s*$/$empty/g };
+
+            # --headings
+            $val = $args->{headings}->($val) if !$row and ref($args->{headings}) eq 'CODE';
+
+            $data->[$row][$col] = _element( $tag => $val, $args->{$tag} );
         }
-
-        $first = 0;
     }
 
-    if (ref($self) and !$self->{is_cached} and $args->{cache}) {
+    if ($args->{cache} and $self and !$self->{is_cached}) {
         $self->{data} = $data;
         $self->{is_cached} = 1;
     }
@@ -100,13 +102,6 @@ sub _make_table {
     return $html;
 }
 
-sub _scrub {
-    my $value = shift;
-    do{ no warnings; $value =~ s/^\s*$/&nbsp;/g };
-    $value =~ s/\n/<br \/>/g;
-    return $value;
-}
-
 sub _element {
     my ($tag, $content, $attr) = @_;
     my $e = HTML::Element->new( $tag, %{ $attr || {} } );
@@ -119,11 +114,8 @@ sub _args {
     $self = shift if UNIVERSAL::isa( $_[0], __PACKAGE__ );
 
     if (@_ > 1 && defined($_[0]) && !ref($_[0]) ) {
-        my %args = @_;
-        if (my $arg = delete $args{data}) {
-            $data = $arg;
-        }
-        $args = {%args};
+        $args = {@_};
+        $data = delete $args->{data} if exists $args->{data};
     } elsif (@_ > 1 && ref($_[0]) eq 'ARRAY') {
         $data = [ @_ ];
     } elsif (@_ == 1) {
@@ -131,38 +123,36 @@ sub _args {
     }
 
     if ($self) {
-        $args = { %{ ref($self) ? $self : {} }, %{ $args || {} } };
+        return ( $self, $self->{data}, $args ) if $self->{is_cached};
+        $args = { %{ $self || {} }, %{ $args || {} } };
         delete $args->{data};
+        $data = $self->{data} unless $data or $args->{file};
     }
 
-    if (ref($self) and $self->{is_cached}) {
-        return ( $self, $self->{data}, $args );
-    }
-
-    unless (exists $args->{file}) {
-        if (ref($self) and !$data) {
-            $data = $self->{data};
-        }
-        $data = [ $data ] unless ref($data);
-        $data = [ $data ] unless ref($data->[0]);
-        $data = [ [undef] ] if !scalar @{ $data->[0] };
-    }
-
-    if (my $file = $args->{file}) {
-        if ($file =~ /\.csv$/) {
-            $data = Spreadsheet::HTML::CSV::load( $file );
-        } elsif ($file =~ /\.html?$/) {
-            $data = Spreadsheet::HTML::HTML::load( $file );
-        } elsif ($file =~ /\.jso?n$/) {
-            $data = Spreadsheet::HTML::JSON::load( $file );
-        } elsif ($file =~ /\.ya?ml$/) {
-            $data = Spreadsheet::HTML::YAML::load( $file );
-        }
-    }
+    $data = _load_file( $args->{file} ) if $args->{file};
+    $data = [ $data ] unless ref($data);
+    $data = [ $data ] unless ref($data->[0]);
+    $data = [ [undef] ] if !scalar @{ $data->[0] };
 
     return ( $self, Clone::clone($data), $args );
 }
 
+sub _load_file {
+    my $file = shift;
+
+    if ($file =~ /\.csv$/) {
+        return Spreadsheet::HTML::CSV::load( $file );
+    } elsif ($file =~ /\.html?$/) {
+        return Spreadsheet::HTML::HTML::load( $file );
+    } elsif ($file =~ /\.jso?n$/) {
+        return Spreadsheet::HTML::JSON::load( $file );
+    } elsif ($file =~ /\.ya?ml$/) {
+        return Spreadsheet::HTML::YAML::load( $file );
+    } elsif ($file =~ /\.xls$/) {
+        return Spreadsheet::HTML::XLS::load( $file );
+    }
+}
+
 1;
 
 __END__
@@ -189,44 +179,33 @@ todo:
 =back
 
 You are encouraged to try my older L<DBIx::XHTML_Table> during
-the development of this module.
+the development of this module, which provides support for 
+tags such as caption, col, colgroup, thead, and tbody.
 
 =head1 SYNOPSIS
 
     use Spreadsheet::HTML;
 
-    my $data = [
-        [qw(header1 header2 header3)],
-        [qw(foo bar baz)],
-        [qw(one two three)],
-        [qw(col1 col2 col3)],
-    ];
+    $data = [ [qw(a1 a2 a3)], [qw(b1 b2 b3)], [qw(c1 c2 c3)] ];
 
-    my $table = Spreadsheet::HTML->new( data => $data, cache => 1 );
-    print $table->generate;
-    print $table->transpose;
-    print $table->flip;
-    print $table->mirror;
-    print $table->reverse;
+    $table = Spreadsheet::HTML->new( data => $data );
+    print $table->portrait;
+    print $table->landscape;
 
     # non OO
-    print Spreadsheet::HTML::generate( $data );
-    print Spreadsheet::HTML::transpose( $data );
-    print Spreadsheet::HTML::flip( $data );
-    print Spreadsheet::HTML::mirror( $data );
-    print Spreadsheet::HTML::reverse( $data );
+    print Spreadsheet::HTML::portrait( $data );
+    print Spreadsheet::HTML::landscape( $data );
+
+    # load from files
+    $table = Spreadsheet::HTML->new( file => 'data.csv', cache => 1 );
 
 =head1 METHODS
 
 =over 4
 
-=item * new( key => 'value' )
+=item * C<new( %args )>
 
-  my $table = Spreadsheet::HTML->new;
-  my $table = Spreadsheet::HTML->new( @data );
-  my $table = Spreadsheet::HTML->new( $data );
   my $table = Spreadsheet::HTML->new( data => $data );
-  my $table = Spreadsheet::HTML->new( { data => $data } );
 
 Constructs object. Accepts named arguments (see ATTRIBUTES).
 Unless you give it an array of array refs. Or an array ref
@@ -235,68 +214,43 @@ most favorite being 'data' which is exactly an array ref
 of array refs. The first row will be treated as the headings
 unless you specify otherwise (see ATTRIBUTES).
 
-=item * process( key => 'value' )
+=item * C<generate( %args )>
 
-Data structure that can be used by the following:
-
-=item * generate( key => 'value' )
-
-  my $html = $table->generate;
-  my $html = $table->generate( indent => '    ' );
-  my $html = $table->generate( encode => '<>&=' );
-  my $html = $table->generate( table => { class => 'foo' } );
-
-  my $html = Spreadsheet::HTML::generate( @data );
-  my $html = Spreadsheet::HTML::generate( $data );
-  my $html = Spreadsheet::HTML::generate(
-      data   => $data,
-      indent => "\t",
-      encode => '<>&=',
-      table  => { class => 'spreadsheet' },
-  );
+  $html = $table->generate( table => {border => 1}, encode => '<>' );
+  print Spreadsheet::HTML::generate( data => $data, indent => "\t" );
 
 Returns a string that contains the rendered HTML table.
-Currently (and subject to change if better ideas arise),
-all data will:
 
-=over 8
+=item * C<portrait( %args )>
 
-=item - be converted to &nbsp; if empty
+Alias for C<generate()>
 
-=item - have any newlines converted to <br> tags
-
-=back
-
-These features are currently hard coded in (sorry). Plans
-to make these transliterations configurable by the client
-are planned. Plans planning plans.
-
-=item * portrait( key => 'value' )
-
-Alias for generate()
-
-=item * transpose( key => 'value' )
+=item * C<transpose( %args )>
 
 Uses Math::Matrix to rotate the headings and data
 90 degrees counter-clockwise.
 
-=item * landscape( key => 'value' )
+=item * C<landscape( %args )>
 
-Alias for transpose()
+Alias for C<transpose()>
 
-=item * flip( key => 'value' )
+=item * C<flip( %args )>
 
 Flips the headings and data upside down.
 
-=item * mirror( key => 'value' )
+=item * C<mirror( %args )>
 
 Columns are rendered right to left.
 
-=item * reverse( key => 'value' )
+=item * C<reverse( %args )>
 
 Combines flip and mirror: flips the headings and
 data upside down and render columns right to left.
 
+=item * C<process( %args )>
+
+Returns processed data.
+
 =back
 
 =head1 ATTRIBUTES
@@ -311,24 +265,24 @@ array references.
 
 =over 4
 
-=item * data => [ [], [], [], ... ]
+=item * C<data: [ [], [], [], ... ]>
 
 The data to be rendered into table cells.
 
-=item * file => $str
+=item * C<file: $str>
 
 The name of the data file to read. Supported formats
 are CSV, JSON, YAML and HTML (first table found).
 Support for Excel files is planned.
 
-=item * indent => $str
+=item * C<indent: $str>
 
 Render the table with whitespace indention. Defaults to
 undefined which produces no trailing whitespace to tags.
 Useful values are some number of spaces or tabs.  (see
 L<HTML::Element>::as_HTML).
 
-=item * encode => $str
+=item * C<encode: $str>
 
 HTML Encode contents of td tags. Defaults to empty string
 which performs no encoding of entities. Pass a string like
@@ -336,25 +290,34 @@ which performs no encoding of entities. Pass a string like
 value is 'undef' then all unsafe characters will be
 encoded as HTML entites (see L<HTML::Element>::as_HTML).
 
-=item * cache => 0 or 1
+=item * C<empty: $str>
+
+Replace empty cells with this value. Defaults to &nbsp;
+Set value to undef to avoid any substitutions.
+
+=item * C<cache: 0 or 1>
 
 Preserve data after it has been processed (and loaded).
 
-=item * matrix => 0 or 1
+=item * C<matrix: 0 or 1>
 
 Render the table with only td tags, no th tags, if true.
 
-=item * headless => 0 or 1
+=item * C<headless: 0 or 1>
 
 Render the table with without headings, if true.
 
-=item * table => { key => 'value' }
+=item * C<headings: sub { return function( shift ) }>
 
-=item * tr => { key => 'value' }
+Apply this anonymous subroutine to headers.
 
-=item * th => { key => 'value' }
+=item * C<table: { %args }>
 
-=item * td => { key => 'value' }
+=item * C<tr: { %args }>
+
+=item * C<th: { %args }>
+
+=item * C<td: { %args }>
 
 Supply attributes to the HTML tags that compose the table.
 There is currently no support for col, colgroup, caption,
@@ -371,11 +334,11 @@ module cannot produce a transposed table, however.
 
 =item * L<HTML::Tree>
 
-Used to generate HTML.
+Used to generate HTML. Handles indentation and HTML entity encoding.
 
 =item * L<Math::Matrix>
 
-Used for transposing data.
+Used for transposing data from portrait to landscape.
 
 =item * L<Clone>
 
@@ -383,14 +346,9 @@ Useful for preventing data from being clobbered.
 
 =back
 
-=head1 REQUIRES (optional)
+=head1 OPTIONAL
 
-These modules are used to load data from various
-different file formats. They should be optional
-but testing is still being conducted on this feature.
-Attempting to parse a file when the necessary module
-is not installed should prompt a clear error and
-suggest action. This version may or may not reflect that. ;)
+Used to load data from various different file formats.
 
 =over 4
 
